@@ -1,30 +1,27 @@
 /*
- * Private MD5 implementation for CUPS.
- *
- * Copyright © 2023 by OpenPrinting.
- * Copyright © 2007-2017 by Apple Inc.
- * Copyright © 2005 by Easy Software Products
- * Copyright © 1999 Aladdin Enterprises.  All rights reserved.
- *
- * This software is provided 'as-is', without any express or implied
- * warranty.  In no event will the authors be held liable for any damages
- * arising from the use of this software.
- *
- * Permission is granted to anyone to use this software for any purpose,
- * including commercial applications, and to alter it and redistribute it
- * freely, subject to the following restrictions:
- *
- * 1. The origin of this software must not be misrepresented; you must not
- *    claim that you wrote the original software. If you use this software
- *    in a product, an acknowledgment in the product documentation would be
- *    appreciated but is not required.
- * 2. Altered source versions must be plainly marked as such, and must not be
- *    misrepresented as being the original software.
- * 3. This notice may not be removed or altered from any source distribution.
- *
- * L. Peter Deutsch
- * ghost@aladdin.com
+  Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+
+  This software is provided 'as-is', without any express or implied
+  warranty.  In no event will the authors be held liable for any damages
+  arising from the use of this software.
+
+  Permission is granted to anyone to use this software for any purpose,
+  including commercial applications, and to alter it and redistribute it
+  freely, subject to the following restrictions:
+
+  1. The origin of this software must not be misrepresented; you must not
+     claim that you wrote the original software. If you use this software
+     in a product, an acknowledgment in the product documentation would be
+     appreciated but is not required.
+  2. Altered source versions must be plainly marked as such, and must not be
+     misrepresented as being the original software.
+  3. This notice may not be removed or altered from any source distribution.
+
+  L. Peter Deutsch
+  ghost@aladdin.com
+
  */
+/*$Id: md5.c,v 1.1.1.4 2002/12/23 18:59:38 jlovell Exp $ */
 /*
   Independent implementation of MD5 (RFC 1321).
 
@@ -41,9 +38,65 @@
   1999-05-03 lpd Original version.
  */
 
-#include "md5-internal.h"
-#include "string-private.h"
+#include "md5.h"
+#include "string.h"
 
+#ifdef TEST
+/*
+ * Compile with -DTEST to create a self-contained executable test program.
+ * The test program should print out the same values as given in section
+ * A.5 of RFC 1321, reproduced below.
+ */
+main()
+{
+    static const char *const test[7] = {
+	"", /*d41d8cd98f00b204e9800998ecf8427e*/
+	"a", /*0cc175b9c0f1b6a831c399e269772661*/
+	"abc", /*900150983cd24fb0d6963f7d28e17f72*/
+	"message digest", /*f96b697d7cb7938d525a2f31aaf161d0*/
+	"abcdefghijklmnopqrstuvwxyz", /*c3fcd3d76192e4007dfb496cca67e13b*/
+	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
+				/*d174ab98d277d9f5a5611c2c9f419d9f*/
+	"12345678901234567890123456789012345678901234567890123456789012345678901234567890" /*57edf4a22be3c955ac49da2e2107b67a*/
+    };
+    int i;
+
+    for (i = 0; i < 7; ++i) {
+	md5_state_t state;
+	md5_byte_t digest[16];
+	int di;
+
+	md5_init(&state);
+	md5_append(&state, (const md5_byte_t *)test[i], strlen(test[i]));
+	md5_finish(&state, digest);
+	printf("MD5 (\"%s\") = ", test[i]);
+	for (di = 0; di < 16; ++di)
+	    printf("%02x", digest[di]);
+	printf("\n");
+    }
+    return 0;
+}
+#endif /* TEST */
+
+
+/*
+ * For reference, here is the program that computed the T values.
+ */
+#if 0
+#include <math.h>
+main()
+{
+    int i;
+    for (i = 1; i <= 64; ++i) {
+	unsigned long v = (unsigned long)(4294967296.0 * fabs(sin((double)i)));
+	printf("#define T%d 0x%08lx\n", i, v);
+    }
+    return 0;
+}
+#endif
+/*
+ * End of T computation program.
+ */
 #define T1 0xd76aa478
 #define T2 0xe8c7b756
 #define T3 0x242070db
@@ -110,15 +163,15 @@
 #define T64 0xeb86d391
 
 static void
-_cups_md5_process(_cups_md5_state_t *pms, const unsigned char *data /*[64]*/)
+md5_process(md5_state_t *pms, const md5_byte_t *data /*[64]*/)
 {
-    unsigned int
+    md5_word_t
 	a = pms->abcd[0], b = pms->abcd[1],
 	c = pms->abcd[2], d = pms->abcd[3];
-    unsigned int t;
+    md5_word_t t;
 
 #ifndef ARCH_IS_BIG_ENDIAN
-#  define ARCH_IS_BIG_ENDIAN 1	/* slower, default implementation */
+# define ARCH_IS_BIG_ENDIAN 1	/* slower, default implementation */
 #endif
 #if ARCH_IS_BIG_ENDIAN
 
@@ -126,13 +179,12 @@ _cups_md5_process(_cups_md5_state_t *pms, const unsigned char *data /*[64]*/)
      * On big-endian machines, we must arrange the bytes in the right
      * order.  (This also works on machines of unknown byte order.)
      */
-    unsigned int X[16];
-    const unsigned char *xp = data;
+    md5_word_t X[16];
+    const md5_byte_t *xp = data;
     int i;
 
     for (i = 0; i < 16; ++i, xp += 4)
-	X[i] = (unsigned)xp[0] + ((unsigned)xp[1] << 8) +
-	       ((unsigned)xp[2] << 16) + ((unsigned)xp[3] << 24);
+	X[i] = xp[0] + (xp[1] << 8) + (xp[2] << 16) + (xp[3] << 24);
 
 #else  /* !ARCH_IS_BIG_ENDIAN */
 
@@ -140,12 +192,12 @@ _cups_md5_process(_cups_md5_state_t *pms, const unsigned char *data /*[64]*/)
      * On little-endian machines, we can process properly aligned data
      * without copying it.
      */
-    unsigned int xbuf[16];
-    const unsigned int *X;
+    md5_word_t xbuf[16];
+    const md5_word_t *X;
 
-    if (!((data - (const unsigned char *)0) & 3)) {
+    if (!((data - (const md5_byte_t *)0) & 3)) {
 	/* data are properly aligned */
-	X = (const unsigned int *)data;
+	X = (const md5_word_t *)data;
     } else {
 	/* not aligned */
 	memcpy(xbuf, data, 64);
@@ -269,7 +321,7 @@ _cups_md5_process(_cups_md5_state_t *pms, const unsigned char *data /*[64]*/)
 }
 
 void
-_cupsMD5Init(_cups_md5_state_t *pms)
+md5_init(md5_state_t *pms)
 {
     pms->count[0] = pms->count[1] = 0;
     pms->abcd[0] = 0x67452301;
@@ -279,18 +331,18 @@ _cupsMD5Init(_cups_md5_state_t *pms)
 }
 
 void
-_cupsMD5Append(_cups_md5_state_t *pms, const unsigned char *data, int nbytes)
+md5_append(md5_state_t *pms, const md5_byte_t *data, int nbytes)
 {
-    const unsigned char *p = data;
+    const md5_byte_t *p = data;
     int left = nbytes;
     int offset = (pms->count[0] >> 3) & 63;
-    unsigned int nbits = (unsigned int)(nbytes << 3);
+    md5_word_t nbits = (md5_word_t)(nbytes << 3);
 
     if (nbytes <= 0)
 	return;
 
     /* Update the message length. */
-    pms->count[1] += (unsigned)nbytes >> 29;
+    pms->count[1] += nbytes >> 29;
     pms->count[0] += nbits;
     if (pms->count[0] < nbits)
 	pms->count[1]++;
@@ -299,42 +351,42 @@ _cupsMD5Append(_cups_md5_state_t *pms, const unsigned char *data, int nbytes)
     if (offset) {
 	int copy = (offset + nbytes > 64 ? 64 - offset : nbytes);
 
-	memcpy(pms->buf + offset, p, (size_t)copy);
+	memcpy(pms->buf + offset, p, copy);
 	if (offset + copy < 64)
 	    return;
 	p += copy;
 	left -= copy;
-	_cups_md5_process(pms, pms->buf);
+	md5_process(pms, pms->buf);
     }
 
     /* Process full blocks. */
     for (; left >= 64; p += 64, left -= 64)
-	_cups_md5_process(pms, p);
+	md5_process(pms, p);
 
     /* Process a final partial block. */
     if (left)
-	memcpy(pms->buf, p, (size_t)left);
+	memcpy(pms->buf, p, left);
 }
 
 void
-_cupsMD5Finish(_cups_md5_state_t *pms, unsigned char digest[16])
+md5_finish(md5_state_t *pms, md5_byte_t digest[16])
 {
-    static const unsigned char pad[64] = {
+    static const md5_byte_t pad[64] = {
 	0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
     };
-    unsigned char data[8];
+    md5_byte_t data[8];
     int i;
 
     /* Save the length before padding. */
     for (i = 0; i < 8; ++i)
-	data[i] = (unsigned char)(pms->count[i >> 2] >> ((i & 3) << 3));
+	data[i] = (md5_byte_t)(pms->count[i >> 2] >> ((i & 3) << 3));
     /* Pad to 56 bytes mod 64. */
-    _cupsMD5Append(pms, pad, (int)((55 - (pms->count[0] >> 3)) & 63) + 1);
+    md5_append(pms, pad, ((55 - (pms->count[0] >> 3)) & 63) + 1);
     /* Append the length. */
-    _cupsMD5Append(pms, data, 8);
+    md5_append(pms, data, 8);
     for (i = 0; i < 16; ++i)
-	digest[i] = (unsigned char)(pms->abcd[i >> 2] >> ((i & 3) << 3));
+	digest[i] = (md5_byte_t)(pms->abcd[i >> 2] >> ((i & 3) << 3));
 }
