@@ -1,11 +1,37 @@
 /*
- * Common filter routines for CUPS.
+ * "$Id: common.c,v 1.1.1.10 2003/04/18 19:52:30 jlovell Exp $"
  *
- * Copyright © 2020-2024 by OpenPrinting.
- * Copyright 2007-2014 by Apple Inc.
- * Copyright 1997-2006 by Easy Software Products.
+ *   Common filter routines for the Common UNIX Printing System (CUPS).
  *
- * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
+ *   Copyright 1997-2003 by Easy Software Products.
+ *
+ *   These coded instructions, statements, and computer programs are the
+ *   property of Easy Software Products and are protected by Federal
+ *   copyright law.  Distribution and use rights are outlined in the file
+ *   "LICENSE.txt" which should have been included with this file.  If this
+ *   file is missing or damaged please contact Easy Software Products
+ *   at:
+ *
+ *       Attn: CUPS Licensing Information
+ *       Easy Software Products
+ *       44141 Airport View Drive, Suite 204
+ *       Hollywood, Maryland 20636-3111 USA
+ *
+ *       Voice: (301) 373-9603
+ *       EMail: cups-info@cups.org
+ *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
+ * Contents:
+ *
+ *   SetCommonOptions() - Set common filter options for media size,
+ *                        etc.
+ *   UpdatePageVars()   - Update the page variables for the orientation.
+ *   WriteCommon()      - Write common procedures...
+ *   WriteLabelProlog() - Write the prolog with the classification
+ *                        and page label.
+ *   WriteLabels()      - Write the actual page labels.
  */
 
 /*
@@ -20,31 +46,30 @@
  * Globals...
  */
 
-int	Orientation = 0,		/* 0 = portrait, 1 = landscape, etc. */
-	Duplex = 0,			/* Duplexed? */
-	LanguageLevel = 1,		/* Language level of printer */
-	ColorDevice = 1;		/* Do color text? */
-float	PageLeft = 18.0f,		/* Left margin */
-	PageRight = 594.0f,		/* Right margin */
-	PageBottom = 36.0f,		/* Bottom margin */
-	PageTop = 756.0f,		/* Top margin */
-	PageWidth = 612.0f,		/* Total page width */
-	PageLength = 792.0f;		/* Total page length */
+int	Orientation = 0,	/* 0 = portrait, 1 = landscape, etc. */
+	Duplex = 0,		/* Duplexed? */
+	LanguageLevel = 1,	/* Language level of printer */
+	ColorDevice = 1;	/* Do color text? */
+float	PageLeft = 18.0f,	/* Left margin */
+	PageRight = 594.0f,	/* Right margin */
+	PageBottom = 36.0f,	/* Bottom margin */
+	PageTop = 756.0f,	/* Top margin */
+	PageWidth = 612.0f,	/* Total page width */
+	PageLength = 792.0f;	/* Total page length */
 
 
 /*
  * 'SetCommonOptions()' - Set common filter options for media size, etc.
  */
 
-ppd_file_t *				/* O - PPD file */
-SetCommonOptions(
-    int           num_options,		/* I - Number of options */
-    cups_option_t *options,		/* I - Options */
-    int           change_size)		/* I - Change page size? */
+ppd_file_t *					/* O - PPD file */
+SetCommonOptions(int           num_options,	/* I - Number of options */
+                 cups_option_t *options,	/* I - Options */
+		 int           change_size)	/* I - Change page size? */
 {
-  ppd_file_t	*ppd;			/* PPD file */
-  ppd_size_t	*pagesize;		/* Current page size */
-  const char	*val;			/* Option value */
+  ppd_file_t	*ppd;		/* PPD file */
+  ppd_size_t	*pagesize;	/* Current page size */
+  const char	*val;		/* Option value */
 
 
 #ifdef LC_TIME
@@ -77,14 +102,9 @@ SetCommonOptions(
 
   if ((val = cupsGetOption("landscape", num_options, options)) != NULL)
   {
-    if (_cups_strcasecmp(val, "no") != 0 && _cups_strcasecmp(val, "off") != 0 &&
-        _cups_strcasecmp(val, "false") != 0)
-    {
-      if (ppd && ppd->landscape > 0)
-        Orientation = 1;
-      else
-        Orientation = 3;
-    }
+    if (strcasecmp(val, "no") != 0 && strcasecmp(val, "off") != 0 &&
+        strcasecmp(val, "false") != 0)
+      Orientation = 1;
   }
   else if ((val = cupsGetOption("orientation-requested", num_options, options)) != NULL)
   {
@@ -202,12 +222,12 @@ SetCommonOptions(
 void
 UpdatePageVars(void)
 {
-  float		temp;			/* Swapping variable */
+  float		temp;		/* Swapping variable */
 
 
   switch (Orientation & 3)
   {
-    case 0 : /* Portrait */
+    case 0 : /* Portait */
         break;
 
     case 1 : /* Landscape */
@@ -302,7 +322,7 @@ WriteLabelProlog(const char *label,	/* I - Page label */
 
   if ((classification = getenv("CLASSIFICATION")) == NULL)
     classification = "";
-  else if (strcmp(classification, "none") == 0)
+  if (strcmp(classification, "none") == 0)
     classification = "";
 
  /*
@@ -338,8 +358,6 @@ WriteLabelProlog(const char *label,	/* I - Page label */
     for (ptr = classification; *ptr; ptr ++)
       if (*ptr < 32 || *ptr > 126)
         printf("\\%03o", *ptr);
-      else if (*ptr == '_')
-        putchar(' ');
       else
       {
 	if (*ptr == '(' || *ptr == ')' || *ptr == '\\')
@@ -442,71 +460,5 @@ WriteLabels(int orient)	/* I - Orientation of the page */
 
 
 /*
- * 'WriteTextComment()' - Write a DSC text comment.
+ * End of "$Id: common.c,v 1.1.1.10 2003/04/18 19:52:30 jlovell Exp $".
  */
-
-void
-WriteTextComment(const char *name,	/* I - Comment name ("Title", etc.) */
-                 const char *value)	/* I - Comment value */
-{
-  int	len;				/* Current line length */
-
-
- /*
-  * DSC comments are of the form:
-  *
-  *   %%name: value
-  *
-  * The name and value must be limited to 7-bit ASCII for most printers,
-  * so we escape all non-ASCII and ASCII control characters as described
-  * in the Adobe Document Structuring Conventions specification.
-  */
-
-  printf("%%%%%s: (", name);
-  len = 5 + (int)strlen(name);
-
-  while (*value)
-  {
-    if (*value < ' ' || *value >= 127)
-    {
-     /*
-      * Escape this character value...
-      */
-
-      if (len >= 251)			/* Keep line < 254 chars */
-        break;
-
-      printf("\\%03o", *value & 255);
-      len += 4;
-    }
-    else if (*value == '\\')
-    {
-     /*
-      * Escape the backslash...
-      */
-
-      if (len >= 253)			/* Keep line < 254 chars */
-        break;
-
-      putchar('\\');
-      putchar('\\');
-      len += 2;
-    }
-    else
-    {
-     /*
-      * Put this character literally...
-      */
-
-      if (len >= 254)			/* Keep line < 254 chars */
-        break;
-
-      putchar(*value);
-      len ++;
-    }
-
-    value ++;
-  }
-
-  puts(")");
-}
