@@ -1,16 +1,25 @@
 /*
- * "$Id: hpgl-input.c 7219 2008-01-14 22:00:02Z mike $"
+ * "$Id: hpgl-input.c,v 1.1.1.7 2002/12/24 00:06:55 jlovell Exp $"
  *
  *   HP-GL/2 input processing for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2007-2008 by Apple Inc.
- *   Copyright 1993-2006 by Easy Software Products.
+ *   Copyright 1993-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ *   property of Easy Software Products and are protected by Federal
+ *   copyright law.  Distribution and use rights are outlined in the file
+ *   "LICENSE.txt" which should have been included with this file.  If this
+ *   file is missing or damaged please contact Easy Software Products
+ *   at:
+ *
+ *       Attn: CUPS Licensing Information
+ *       Easy Software Products
+ *       44141 Airport View Drive, Suite 204
+ *       Hollywood, Maryland 20636-3111 USA
+ *
+ *       Voice: (301) 373-9603
+ *       EMail: cups-info@cups.org
+ *         WWW: http://www.cups.org
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -26,7 +35,6 @@
 
 #include "hpgltops.h"
 #include <ctype.h>
-#include <cups/i18n.h>
 
 #define MAX_PARAMS 16384
 
@@ -46,9 +54,7 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
 		ch,		/* Current char */
 		done,		/* Non-zero when the current command is read */
 		i;		/* Looping var */
-  char		buf[262144],	/* String buffer */
-		*bufptr;	/* Pointer into buffer */
-  float		temp;		/* Temporary parameter value */
+  char		buf[262144];	/* String buffer */
   static param_t p[MAX_PARAMS];	/* Parameter buffer */
 
 
@@ -62,9 +68,7 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
         break;
 
     if (ch == EOF)
-    {
       return (-1);
-    }
 
     if (ch == 0x1b)
       switch (getc(fp))
@@ -99,61 +103,26 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
             }
             break;
 
-        case '%' : /* PJL command? */
-            if ((i = getc(fp)) == '-')
-	      if ((i = getc(fp)) == '1')
-	        if ((i = getc(fp)) == '2')
-		{
-		 /*
-		  * Yes, dump everything up to the "ENTER LANGUAGE" line...
-		  */
-
-        	  while (fgets(buf, sizeof(buf), fp) != NULL)
-	            if (strstr(buf, "ENTER") && strstr(buf, "LANGUAGE"))
-		      break;
-		  break;
-		}
-
-            ungetc(i, fp);
-
         default : /* HP RTL/PCL control */
-            while ((i = getc(fp)) != EOF && !isupper(i & 255));
-
-	    if (i == EOF)
-	      return (-1);
+            while ((i = getc(fp)) != EOF && !isupper(i));
             break;
       }
-  } while (ch < ' ');
+  } while (ch == 0x1b);
 
   name[0] = ch;
   name[1] = getc(fp);
   name[2] = '\0';
 
-  if (name[1] < ' ')
+  if (strcasecmp(name, "LB") == 0)
   {
-   /*
-    * If we get here, then more than likely we are faced with a raw PCL
-    * file which we can't handle - abort!
-    */
-
-    fputs(_("ERROR: Invalid HP-GL/2 command seen, unable to print file!\n"),
-          stderr);
-    return (-1);
-  }
-
-  if (!strcasecmp(name, "LB"))
-  {
-    bufptr = buf;
-    while ((ch = getc(fp)) != StringTerminator)
-      if (bufptr < (buf + sizeof(buf) - 1))
-        *bufptr++ = ch;
-    *bufptr = '\0';
-
+    for (i = 0; (ch = getc(fp)) != StringTerminator; i ++)
+      buf[i] = ch;
+    buf[i] = '\0';
     p[num_params].type         = PARAM_STRING;
     p[num_params].value.string = strdup(buf);
     num_params ++;
   }
-  else if (!strcasecmp(name, "SM"))
+  else if (strcasecmp(name, "SM") == 0)
   {
     buf[0] = getc(fp);
     buf[1] = '\0';
@@ -161,7 +130,7 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
     p[num_params].value.string = strdup(buf);
     num_params ++;
   }
-  else if (!strcasecmp(name, "DT"))
+  else if (strcasecmp(name, "DT") == 0)
   {
     if ((buf[0] = getc(fp)) != ';')
     {
@@ -171,16 +140,13 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
       num_params ++;
     }
   }
-  else if (!strcasecmp(name, "PE"))
+  else if (strcasecmp(name, "PE") == 0)
   {
-    bufptr = buf;
-    while ((ch = getc(fp)) != ';')
-      if (ch == EOF)
+    for (i = 0; i < (sizeof(buf) - 1); i ++)
+      if ((buf[i] = getc(fp)) == ';')
         break;
-      else if (bufptr < (buf + sizeof(buf) - 1))
-        *bufptr++ = ch;
-    *bufptr = '\0';
 
+    buf[i] = '\0';
     p[num_params].type         = PARAM_STRING;
     p[num_params].value.string = strdup(buf);
     num_params ++;
@@ -189,10 +155,6 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
   while (!done)
     switch (ch = getc(fp))
     {
-      case EOF :
-          done = 1;
-          break;
-
       case ',' :
       case ' ' :
       case '\n' :
@@ -213,10 +175,10 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
       case '-' :
       case '+' :
           ungetc(ch, fp);
-          if (fscanf(fp, "%f", &temp) == 1 && num_params < MAX_PARAMS)
+          fscanf(fp, "%f", &(p[num_params].value.number));
+          if (num_params < MAX_PARAMS)
           {
-            p[num_params].type         = PARAM_RELATIVE;
-            p[num_params].value.number = temp;
+            p[num_params].type = PARAM_RELATIVE;
             num_params ++;
           }
           break;
@@ -232,10 +194,10 @@ ParseCommand(FILE    *fp,	/* I - File to read from */
       case '9' :
       case '.' :
           ungetc(ch, fp);
-          if (fscanf(fp, "%f", &temp) == 1 && num_params < MAX_PARAMS)
+          fscanf(fp, "%f", &(p[num_params].value.number));
+          if (num_params < MAX_PARAMS)
           {
-            p[num_params].type         = PARAM_ABSOLUTE;
-            p[num_params].value.number = temp;
+            p[num_params].type = PARAM_ABSOLUTE;
             num_params ++;
           }
           break;
@@ -268,5 +230,5 @@ FreeParameters(int     num_params,	/* I - Number of parameters */
 
 
 /*
- * End of "$Id: hpgl-input.c 7219 2008-01-14 22:00:02Z mike $".
+ * End of "$Id: hpgl-input.c,v 1.1.1.7 2002/12/24 00:06:55 jlovell Exp $".
  */
