@@ -1,17 +1,26 @@
 /*
- * "$Id: image-sgi.h 6649 2007-07-11 21:46:42Z mike $"
+ * "$Id: image-sgi.h,v 1.1.1.7 2002/12/24 00:07:02 jlovell Exp $"
  *
  *   SGI image file format library definitions for the Common UNIX Printing
  *   System (CUPS).
  *
- *   Copyright 2007 by Apple Inc.
- *   Copyright 1993-2005 by Easy Software Products.
+ *   Copyright 1993-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ *   property of Easy Software Products and are protected by Federal
+ *   copyright law.  Distribution and use rights are outlined in the file
+ *   "LICENSE.txt" which should have been included with this file.  If this
+ *   file is missing or damaged please contact Easy Software Products
+ *   at:
+ *
+ *       Attn: CUPS Licensing Information
+ *       Easy Software Products
+ *       44141 Airport View Drive, Suite 204
+ *       Hollywood, Maryland 20636-3111 USA
+ *
+ *       Voice: (301) 373-9603
+ *       EMail: cups-info@cups.org
+ *         WWW: http://www.cups.org
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  */
@@ -71,7 +80,7 @@ typedef struct
 
 extern int	sgiClose(sgi_t *sgip);
 extern int	sgiGetRow(sgi_t *sgip, unsigned short *row, int y, int z);
-extern sgi_t	*sgiOpen(const char *filename, int mode, int comp, int bpp,
+extern sgi_t	*sgiOpen(char *filename, int mode, int comp, int bpp,
 		         int xsize, int ysize, int zsize);
 extern sgi_t	*sgiOpenFile(FILE *file, int mode, int comp, int bpp,
 		             int xsize, int ysize, int zsize);
@@ -83,5 +92,5 @@ extern int	sgiPutRow(sgi_t *sgip, unsigned short *row, int y, int z);
 #endif /* !_SGI_H_ */
 
 /*
- * End of "$Id: image-sgi.h 6649 2007-07-11 21:46:42Z mike $".
+ * End of "$Id: image-sgi.h,v 1.1.1.7 2002/12/24 00:07:02 jlovell Exp $".
  */
